@@ -1,6 +1,7 @@
 """AJAX views for the Curate app
 """
 import json
+import logging
 
 from django.contrib import messages
 from django.contrib.messages.storage.base import Message
@@ -12,6 +13,7 @@ from lxml.etree import XMLSyntaxError
 
 import core_curate_app.common.exceptions as exceptions
 import core_curate_app.components.curate_data_structure.api as curate_data_structure_api
+from core_main_app.components.template import api as template_api
 import core_curate_app.permissions.rights as rights
 import core_curate_app.views.user.forms as users_forms
 import core_main_app.utils.decorators as decorators
@@ -32,6 +34,8 @@ from core_parser_app.tools.parser.parser import remove_child_element
 from core_parser_app.tools.parser.renderer.list import ListRenderer
 from xml_utils.xsd_tree.xsd_tree import XSDTree
 
+logger = logging.getLogger(__name__)
+
 
 # FIXME: delete_branch not deleting all elements
 # FIXME: generate element not testing max occurrences
@@ -82,12 +86,13 @@ def generate_choice(request, curate_data_structure_id):
         curate_data_structure = curate_data_structure_api.get_by_id(
             curate_data_structure_id, request.user
         )
-        xsd_parser = get_parser()
-        html_form = xsd_parser.generate_choice_absent(
-            request, element_id, curate_data_structure.template.content
+        xsd_parser = get_parser(request=request)
+        template = template_api.get(
+            str(curate_data_structure.template.id), request=request
         )
+        html_form = xsd_parser.generate_choice_absent(element_id, template.content)
     except Exception as e:
-        return HttpResponseBadRequest()
+        return HttpResponseBadRequest(escape(str(e)))
 
     return HttpResponse(html_form)
 
@@ -112,12 +117,13 @@ def generate_element(request, curate_data_structure_id):
         curate_data_structure = curate_data_structure_api.get_by_id(
             curate_data_structure_id, request.user
         )
-        xsd_parser = get_parser()
-        html_form = xsd_parser.generate_element_absent(
-            request, element_id, curate_data_structure.template.content
+        xsd_parser = get_parser(request=request)
+        template = template_api.get(
+            str(curate_data_structure.template.id), request=request
         )
+        html_form = xsd_parser.generate_element_absent(element_id, template.content)
     except Exception as e:
-        return HttpResponseBadRequest()
+        return HttpResponseBadRequest(escape(str(e)))
 
     return HttpResponse(html_form)
 
@@ -138,7 +144,7 @@ def remove_element(request):
 
     """
     element_id = request.POST["id"]
-    element_list = data_structure_element_api.get_all_by_child_id(element_id)
+    element_list = data_structure_element_api.get_all_by_child_id(element_id, request)
 
     if len(element_list) == 0:
         raise ValueError("No Data Structure Element found")
@@ -147,13 +153,15 @@ def remove_element(request):
 
     # Removing the element from the data structure
     data_structure_element = element_list[0]
-    data_structure_element_to_pull = data_structure_element_api.get_by_id(element_id)
+    data_structure_element_to_pull = data_structure_element_api.get_by_id(
+        element_id, request
+    )
 
     # number of children after deletion
     children_number = len(data_structure_element.children) - 1
 
     data_structure_element = remove_child_element(
-        data_structure_element, data_structure_element_to_pull
+        data_structure_element, data_structure_element_to_pull, request
     )
 
     response = {"code": 0, "html": ""}
@@ -195,7 +203,10 @@ def clear_fields(request):
         )
 
         # generate form
-        root_element = generate_form(curate_data_structure.template.content)
+        template = template_api.get(
+            str(curate_data_structure.template.id), request=request
+        )
+        root_element = generate_form(template.content, request=request)
 
         # save the root element in the data structure
         curate_data_structure_api.update_data_structure_root(
@@ -208,8 +219,8 @@ def clear_fields(request):
         return HttpResponse(
             json.dumps({"xsdForm": xsd_form}), content_type="application/javascript"
         )
-    except:
-        return HttpResponseBadRequest()
+    except Exception as e:
+        return HttpResponseBadRequest(escape(str(e)))
 
 
 @decorators.permission_required(
@@ -244,7 +255,10 @@ def cancel_changes(request):
             xml_data = None
 
         # generate form
-        root_element = generate_form(curate_data_structure.template.content, xml_data)
+        template = template_api.get(
+            str(curate_data_structure.template.id), request=request
+        )
+        root_element = generate_form(template.content, xml_data, request=request)
 
         # save the root element in the data structure
         curate_data_structure_api.update_data_structure_root(
@@ -257,8 +271,8 @@ def cancel_changes(request):
         return HttpResponse(
             json.dumps({"xsdForm": xsd_form}), content_type="application/javascript"
         )
-    except:
-        return HttpResponseBadRequest()
+    except Exception as e:
+        return HttpResponseBadRequest(escape(str(e)))
 
 
 @decorators.permission_required(
@@ -294,8 +308,11 @@ def cancel_form(request):
         )
 
         return HttpResponse(json.dumps({}), content_type="application/javascript")
-    except:
-        return HttpResponseBadRequest()
+    except Exception as e:
+        return HttpResponseBadRequest(
+            "An unexpected error has occured: %s" % str(e).replace('"', "'"),
+            content_type="application/javascript",
+        )
 
 
 @decorators.permission_required(
@@ -320,7 +337,9 @@ def save_form(request):
         )
 
         # generate xml data
-        xml_data = render_xml(curate_data_structure.data_structure_element_root)
+        xml_data = render_xml(
+            request, curate_data_structure.data_structure_element_root
+        )
 
         # update curate data structure data
         curate_data_structure.form_string = xml_data
@@ -337,8 +356,8 @@ def save_form(request):
             json.dumps({"message": message.message, "tags": message.tags}),
             content_type="application/json",
         )
-    except:
-        return HttpResponseBadRequest()
+    except Exception as e:
+        return HttpResponseBadRequest(escape(str(e)))
 
 
 @decorators.permission_required(
@@ -364,14 +383,19 @@ def validate_form(request):
         )
 
         # generate the XML
-        xml_data = render_xml(curate_data_structure.data_structure_element_root)
+        xml_data = render_xml(
+            request, curate_data_structure.data_structure_element_root
+        )
 
         # build trees
-        xsd_tree = XSDTree.build_tree(curate_data_structure.template.content)
+        template = template_api.get(
+            str(curate_data_structure.template.id), request=request
+        )
+        xsd_tree = XSDTree.build_tree(template.content)
         xml_tree = XSDTree.build_tree(xml_data)
 
         # validate XML document
-        errors = validate_xml_data(xsd_tree, xml_tree)
+        errors = validate_xml_data(xsd_tree, xml_tree, request=request)
 
         if errors is not None:
             response_dict["errors"] = errors
@@ -419,7 +443,9 @@ def save_data(request):
             lock_api.remove_lock_on_object(curate_data_structure.data, request.user)
 
         # generate the XML
-        xml_data = render_xml(curate_data_structure.data_structure_element_root)
+        xml_data = render_xml(
+            request, curate_data_structure.data_structure_element_root
+        )
 
         if curate_data_structure.data is not None:
             # update existing data
@@ -428,13 +454,16 @@ def save_data(request):
             # create new data
             data = Data()
             data.title = curate_data_structure.name
-            data.template = curate_data_structure.template
+            template = template_api.get(
+                str(curate_data_structure.template.id), request=request
+            )
+            data.template = template
             data.user_id = str(request.user.id)
 
         # set content
         data.xml_content = xml_data
         # save data
-        data = data_api.upsert(data, request.user)
+        data = data_api.upsert(data, request)
 
         curate_data_structure_api.delete(curate_data_structure, request.user)
 
@@ -444,8 +473,9 @@ def save_data(request):
             get_data_label().capitalize() + " saved with success.",
         )
     except Exception as e:
-        message = str(e).replace('"', "'")
-        return HttpResponseBadRequest(message, content_type="application/javascript")
+        return HttpResponseBadRequest(
+            str(e).replace('"', "'"), content_type="application/javascript"
+        )
 
     return HttpResponse(
         json.dumps({"data_id": str(data.id)}), content_type="application/javascript"
@@ -485,7 +515,8 @@ def _start_curate_post(request):
                 name = xml_file.name
                 if not well_formed:
                     raise CurateAjaxError(
-                        "An error occurred during the file upload: the file is not well formed XML"
+                        "An error occurred during the file upload: the file is "
+                        "not well formed XML"
                     )
                 else:
                     curate_data_structure = CurateDataStructure(
@@ -545,6 +576,7 @@ def _start_curate_get(request):
             content_type="application/javascript",
         )
     except Exception as e:
+        logger.error(str(e))
         raise exceptions.CurateAjaxError(
             "Error occurred during the " + get_form_label() + " display."
         )
